# encoding: UTF-8
#
# Copyright 2015, Ole Claussen <claussen.ole@gmail.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

#<
# Creates the Bash configuration file for a user, by default `~/.bashrc`.
#>

include_resource 'mixin/user_info'
include_resource 'mixin/dotfile'
include_resource 'mixin/shell'

resource_name :bash_dotfile

provides :dotfile do |node|
  node.run_context.resource_builder.name =~ regex_from_app_name('bash')
end

def initialize(name, run_context = nil)
  super(name, run_context)

  target '.bashrc'
  template_source 'bash/bashrc.erb'
end

class BashOption
  def initialize(name)
    @name = name
  end

  def to_s
    if options_using_set.include? @name
      "set -o #{@name}"
    else
      "shopt -s #{@name}"
    end
  end

  private

  def options_using_set
    [
      'brace exponsion', 'noclobber', 'allexport',
      'notify', 'errexit', 'noglob', 'interactive',
      'noexec', 'POSIX', 'pipe failure', 'privileged',
      'restricted', 'stdin', 'nounset', 'verbose', 'xtrace',
      'ignoreeof'
    ]
  end
end

#<
# @section Bash Prompt
# The `bash_dotfile` supports the Prompt Builder and adds the following options:
#
# * `date`: The date in "Weekday Month Date" format (e.g., "Tue May 26")
# * `hostname_short`: The hostname up to the first `.'
# * `hostname_long`: The full hostname
# * `hostname`: The full hostname (equivalent to `hostname_long`)
# * `jobs`: The number of jobs currently managed by the shell
# * `tty`: The basename of the shell's terminal device name
# * `shell`: The name of the shell
# * `time`: The current time in 24-hour HH:MM:SS format
# * `time_24h`: The current time in 24-hour HH:MM:SS format
# * `time_12h`: The current time in 12-hour HH:MM:SS format
# * `time_am_pm`: The current time in 12-hour am/pm format
# * `user`: The username of the current user
# * `version`: The version of bash (e.g., 2.00)
# * `version_long`: The release of bash, version + patch level (e.g., 2.00.0)
# * `cwd`: The current working directory, with $HOME abbreviated with a tilde
# * `cwd_base`: The basename of the current working directory, with $HOME abbreviated with a tilde
# * `history`: The history number of this command
# * `command`: The command number of this command
# * `pound`: If the effective UID is 0, a #, otherwise a $
#>
class BashPrompt < Prompt
  def noprint(text)
    "\\[#{text}\\]"
  end

  provide_escape_sequence :date, '\d'
  provide_escape_sequence :hostname_short, '\h'
  provide_escape_sequence :hostname_long, '\H'
  provide_escape_sequence :hostname, '\H'
  provide_escape_sequence :jobs, '\j'
  provide_escape_sequence :tty, '\l'
  provide_escape_sequence :shell, '\s'
  provide_escape_sequence :time, '\t'
  provide_escape_sequence :time_24h, '\t'
  provide_escape_sequence :time_12h, '\T'
  provide_escape_sequence :time_am_pm, '\@'
  provide_escape_sequence :user, '\u'
  provide_escape_sequence :version, '\v'
  provide_escape_sequence :version_long, '\V'
  provide_escape_sequence :cwd, '\w'
  provide_escape_sequence :cwd_base, '\W'
  provide_escape_sequence :history, '\!'
  provide_escape_sequence :command, '\#'
  provide_escape_sequence :pound, '\$'
end

#<> @property bash_it @args (resource) Include Bash It configuration.
property :bash_it, ::Chef::CookieCutter::FancyProperty,
         coerce_resource: :bash_it_dotfile

#<> @property prompt @args (block) Configure the Bash prompt (see [Prompt Builder](#bash-prompt)).
property :prompt, ::Chef::CookieCutter::FancyProperty,
         coerce_class: BashPrompt

#<> @property option @args (name) Set either a [Bash option](https://www.gnu.org/software/bash/manual/html_node/The-Set-Builtin.html) or [Shell option](http://tldp.org/LDP/abs/html/options.html).
property :option, ::Chef::CookieCutter::FancyProperty,
         collect: true, coerce_class: BashOption
