# encoding: UTF-8
#
# Copyright 2015, Ole Claussen <claussen.ole@gmail.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

#<
# Creates a [Git](https://git-scm.com) configuration file, by default
# `~/.gitconfig`.
#>

include_resource 'mixin/user_info'
include_resource 'mixin/dotfile'

resource_name :git_dotfile

provides :dotfile do |node|
  node.run_context.resource_builder.name =~ regex_from_app_name('git')
end

def initialize(name, run_context = nil)
  super(name, run_context)

  target '.gitconfig'
  template_source 'git/gitconfig.erb'
end

class GitURL
  def initialize(url, **options)
    @url = url
    @options = options
  end

  def option(key, value)
    @options[key] = value
  end

  def to_s
    result = []
    result << "[url \"#{@url}\"]"
    @options.each do |key, value|
      result << "  #{key} = #{value}"
    end
    result.join("\n")
  end
end

class GitAlias
  def initialize(name, command)
    @name = name
    @command = command
  end

  def to_s
    "#{@name} = #{@command}"
  end
end

class GitColor
  attr_reader :type

  def initialize(type, name, foreground: nil, background: nil, attribute: nil)
    @type = type
    @name = name
    unless foreground.nil?
      @foreground = ::Chef::Dotfiles::Colors.from_property foreground
    end
    unless background.nil?
      @background = ::Chef::Dotfiles::Colors.from_property background
    end
    @attribute = attribute unless attribute.nil?
  end

  %w(foreground background attribute).each do |method|
    define_method("#{method}?") do
      !instance_variable_get("@#{method}").nil?
    end
    attr_reader method
  end

  def to_s
    values = []
    values << "#{@name} ="
    values << foreground.to_termcolor if foreground?
    values << background.to_termcolor if background?
    values << attribute.to_s if attribute?
    values.join(' ')
  end
end

class GitOption
  attr_reader :section
  attr_reader :subsection

  def initialize(section, subsection = nil, name, value) # rubocop:disable Style/OptionalArguments
    @section = section
    @subsection = subsection
    @name = name
    @value = value
  end

  def to_s
    "#{@name} = #{@value}"
  end
end

#<> @property url @args (name, block) Define a URL option. Can be passed an additional block with options.
property :url, ::Chef::CookieCutter::FancyProperty, collect: true, coerce_class: GitURL

#<> @property git_alias @args (name, command) Define a Git alias.
property :git_alias, ::Chef::CookieCutter::FancyProperty, collect: true, coerce_class: GitAlias

#<> @property color @args (type, name, foreground: nil, background: nil, attribute: nil) Define a color option.
property :color, ::Chef::CookieCutter::FancyProperty, collect: true, coerce_class: GitColor

#<> @property option @args (section, subsection=nil, name, value) Set a Git option.
property :option, ::Chef::CookieCutter::FancyProperty, collect: true, coerce_class: GitOption

#<> @property include_path @args (path) Include an additional config file.
property :include_path, ::Chef::CookieCutter::FancyProperty, collect: true, kind_of: String

def colors_by_type
  result = {}
  color.each do |c|
    result[c.type] = [] if result[c.type].nil?
    result[c.type] << c
  end
  result
end

def options_by_section
  result = {}
  option.each do |o|
    result[o.section] = {} if result[o.section].nil?
    result[o.section][o.subsection] = [] if result[o.section][o.subsection].nil?
    result[o.section][o.subsection] << o
  end
  result
end
