# encoding: UTF-8
#
# Copyright 2015, Ole Claussen <claussen.ole@gmail.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

#<
# @mixin mixin/dotfile
# This is the base resource for dotfiles. It declares default actions and
# common properties for all dotfiles. A dotfile can have various sources
# (template, file or Git repository).
# In addition, a dotfile can create additional required directories, or
# perform additional actions in form of Chef resources.
#>

def self.regex_from_app_name(name)
  Regexp.new(
    "^\.?#{Regexp.escape(name)}([-\._ /]?((rc)|(conf(ig)?)))?$",
    Regexp::IGNORECASE
  )
end

def regex_from_app_name(name)
  self.class.regex_from_app_name(name)
end

default_action :create
allowed_actions :create

class DotfileFileSource
  attr_reader :name
  attr_reader :cookbook

  def initialize(name, cookbook: 'dotfiles')
    @name = name
    @cookbook = cookbook
  end
end

class DotfileGitSource
  attr_reader :repository
  attr_reader :reference

  def initialize(repository, reference: 'master')
    @repository = repository
    @reference = reference
  end
end

#<> @property target @args (target)  The file or directory created, implementing the dotfile.
property :target, kind_of: String, name_property: true

#<> @property executable @args (executable) Whether the dotfile should be executable.
property :executable, kind_of: [TrueClass, FalseClass], default: false

#<> @property template_source @args (name, cookbook: 'dotfiles') Configure a template as the dotfile source.
property :template_source, ::Chef::CookieCutter::FancyProperty, coerce_class: DotfileFileSource

#<> @property file_source @args (name, cookbook: 'dotfiles') Configure a cookbook file as the dotfile source.
property :file_source, ::Chef::CookieCutter::FancyProperty, coerce_class: DotfileFileSource

#<> @property remote_source @args (name)  Configure a remote file as the dotfile source.
property :remote_source, kind_of: String

#<> @property git_source @args (repository, reference: 'master') Configure a Git repository as the dotfile source.
property :git_source, ::Chef::CookieCutter::FancyProperty, coerce_class: DotfileGitSource

#<> @property create_directory @args (name) Creates a directory relative to the home directory.
property :create_directory, ::Chef::CookieCutter::FancyProperty, collect: true

#<> @property additional_action @args (action) A proc defining an additional action to be performed.
property :additional_action, ::Chef::CookieCutter::FancyProperty, kind_of: Proc, collect: true

#<> @action create Creates the dotfile
action :create do
  new_resource = self

  new_resource.create_directory.each do |dir|
    directory ::File.join(new_resource.home, dir) do
      action :create
      recursive true
      owner new_resource.user
      group new_resource.group
      mode '0755'
    end
  end

  unless new_resource.template_source.nil?
    template ::File.join(new_resource.home, new_resource.target) do
      action :create
      source new_resource.template_source.name
      cookbook new_resource.template_source.cookbook
      owner new_resource.user
      group new_resource.group
      mode(new_resource.executable ? '0755' : '0644')
      variables(
        cookbook_name: cookbook_name,
        resource: new_resource
      )
    end
  end

  unless new_resource.file_source.nil?
    cookbook_file ::File.join(new_resource.home, new_resource.target) do
      action :create
      source new_resource.file_source.name
      cookbook new_resource.file_source.cookbook
      owner new_resource.owner
      group new_resource.group
      mode(new_resource.executable ? '0755' : '0644')
    end
  end

  unless new_resource.remote_source.nil?
    remote_file ::File.join(new_resource.home, new_resource.target) do
      action :create
      source new_resource.remote_file
      owner new_resource.owner
      group new_resource.group
      mode(new_resource.executable ? '0755' : '0644')
    end
  end

  unless new_resource.git_source.nil?
    directory ::File.join(new_resource.home, new_resource.target) do
      action :create
      recursive true
      user new_resource.user
      group new_resource.group
      mode '0755'
    end

    git ::File.join(new_resource.home, new_resource.target) do
      action :sync
      repository new_resource.git_source.repository
      reference new_resource.git_source.reference
      user new_resource.user
      group new_resource.group
    end
  end

  new_resource.additional_action.each do |act|
    instance_exec(new_resource, &act) if act.is_a?(Proc)
  end
end
