# encoding: UTF-8
#
# Copyright 2015, Ole Claussen <claussen.ole@gmail.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

#<
# @mixin mixin/shell
# This is the base resources for shell dotfiles. It adds some common properties
# for shells.
#
# @section Prompt Builder
# The dotfiles cookbook contains a prompt builder for shell dotfiles, to
# simplify the definition of custom prompts. The prompt builder is extended by
# the individual shell resources. By default, it includes functions to set the
# foreground and background colors and style of prompt segments.
#
# Example:
#
# ```ruby
# prompt do
#   segment red('foo'), blue('bar'), clear
# end
# ```
#>

class ShellAlias
  def initialize(name, command)
    @name = name
    @command = command
  end

  def to_s
    "alias #{@name}=\"#{@command}\""
  end
end

class EnvironmentVariable
  def initialize(name, value)
    @name = name
    @value = value
  end

  def to_s
    "export #{@name}=\"#{@value}\""
  end
end

class Prompt
  FOREGROUNDS ||= {
    enable: { black: 30, red: 31, green: 32, yellow: 33, blue: 34, magenta: 35, cyan: 36, white: 37 },
    disable: 38
  }

  BACKGROUNDS ||= {
    enable: { black: 40, red: 41, green: 42, yellow: 43, blue: 44, magenta: 45, cyan: 46, white: 47 },
    disable: 48
  }

  ATTRIBUTES ||= {
    enable: { bold: 1, underline: 4, blink: 5, reverse: 7, concealed: 8 },
    disable: { bold: 22, underline: 24, blink: 25, reverse: 27, concealed: 28 }
  }

  def self.provide_escape_sequence(name, text = nil, &blk)
    if text.nil? && block_given?
      define_method name do |*args, **kwargs|
        PromptSegment.new(instance_exec(*args, **kwargs, &blk))
      end
    else
      define_method name do
        PromptSegment.new(text)
      end
    end
  end

  def initialize(*args)
    @segments = []
    segment(args) unless args.empty?
  end

  def segment(*args, **kwargs)
    @segments << PromptSegment.new(*args, **kwargs, noprint: method(:noprint))
  end

  def to_s
    @segments.map(&:to_s).join
  end

  def noprint(text)
    ::Chef::Log.warn 'No noprint escape sequence defined for prompt.'
    text
  end

  provide_escape_sequence(:literal) { |text| text }
  provide_escape_sequence(:clear) { noprint '\033[0m' }

  def colored(*args, **kwargs)
    PromptSegment.new(*args, **kwargs, noprint: method(:noprint))
  end

  FOREGROUNDS[:enable].each do |name, _|
    define_method name do |*args|
      colored(*args, foreground: name)
    end
  end

  BACKGROUNDS[:enable].each do |name, _|
    define_method "#{name}_background".to_sym do |*args|
      colored(*args, background: name)
    end
  end

  ATTRIBUTES[:enable].each do |name, _|
    define_method name do |*args|
      colored(*args, attribute: name)
    end
  end

  class PromptSegment
    def initialize(*args, noprint: nil, foreground: nil, background: nil, attribute: nil)
      @children = []
      @foreground = foreground
      @background = background
      @attribute = attribute
      @noprint = noprint
      args.each do |arg|
        @children << arg if arg.is_a?(String) || arg.is_a?(PromptSegment)
      end
    end

    attr_reader :foreground
    attr_reader :background
    attr_reader :attribute

    def foreground?
      !@foreground.nil? || @children.any? do |child|
        child.is_a?(PromptSegment) && child.foreground?
      end
    end

    def background?
      !@background.nil? || @children.any? do |child|
        child.is_a?(PromptSegment) && child.background?
      end
    end

    def colored(foreground: nil, background: nil, attribute: nil)
      @foreground = foreground unless foreground.nil?
      @background = background unless background.nil?
      @attribute = attribute unless attribute.nil?
      self
    end

    FOREGROUNDS[:enable].each do |name, _|
      define_method name do
        colored(foreground: name)
      end
    end

    BACKGROUNDS[:enable].each do |name, _|
      define_method "#{name}_background".to_sym do
        colored(background: name)
      end
    end

    ATTRIBUTES[:enable].each do |name, _|
      define_method name do
        colored attribute: name
      end
    end

    def to_s
      segments = []
      segments << enable_code
      segments << @children.map { |child| child.to_s + rescue_code(child) }
      segments << disable_code
      segments.join
    end

    private

    def enable_code
      codes = []
      codes << FOREGROUNDS[:enable][@foreground] unless @foreground.nil?
      codes << BACKGROUNDS[:enable][@background] unless @background.nil?
      codes << ATTRIBUTES[:enable][@attribute] unless @attribute.nil?
      return '' if codes.empty?
      @noprint.call "\\033[#{codes.join ';'}m"
    end

    def disable_code
      codes = []
      codes << FOREGROUNDS[:disable] unless @foreground.nil?
      codes << BACKGROUNDS[:disable] unless @background.nil?
      codes << ATTRIBUTES[:disable][@attribute] unless @attribute.nil?
      return '' if codes.empty?
      @noprint.call "\\033[#{codes.join ';'}m"
    end

    def rescue_code(child)
      return '' if child.is_a?(String)
      codes = []
      codes << FOREGROUNDS[:enable][@foreground] if !@foreground.nil? && child.foreground?
      codes << BACKGROUNDS[:enable][@background] if !@background.nil? && child.background?
      return '' if codes.empty?
      @noprint.call "\\033[#{codes.join ';'}m"
    end
  end
end

#<> @property shell_alias @args (name, command) Define a shell alias.
property :shell_alias, ::Chef::CookieCutter::FancyProperty,
         collect: true, coerce_class: ShellAlias

#<> @property variable @args (name, value) Export an environment variable.
property :variable, ::Chef::CookieCutter::FancyProperty,
         collect: true, coerce_class: EnvironmentVariable

#<> @property path @args (directory) Add a directory to the PATH environment variable.
property :path, ::Chef::CookieCutter::FancyProperty,
         collect: true, kind_of: String

#<> @property source_file @args (file) Source an external file for custom configuration.
property :source_file, ::Chef::CookieCutter::FancyProperty,
         collect: true, kind_of: String

#<> @property script @args (text) Include a snippet of plain shell script.
property :script, ::Chef::CookieCutter::FancyProperty,
         collect: true, kind_of: String
