# encoding: UTF-8
#
# Copyright 2015, Ole Claussen <claussen.ole@gmail.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

#<
# Creates a [tmux](https://tmux.github.io) configuration file, by default
# `~/.tmux.conf`.
#>

include_resource 'mixin/user_info'
include_resource 'mixin/dotfile'

resource_name :tmux_dotfile

provides :dotfile do |node|
  node.run_context.resource_builder.name =~ regex_from_app_name('tmux')
end

def initialize(name, run_context = nil)
  super(name, run_context)

  target '.tmux.conf'
  template_source 'tmux/tmux.conf.erb'
end

class TmuxOption
  def initialize(name, value = true, scope: '')
    @name = name
    @value = value
    @scope = scope
    @scope = 'w' if scope == 'window'
    @scope = 's' if scope == 'server'
  end

  def to_s
    "set-option -#{@scope}g #{@name} #{@value}"
  end
end

class TmuxKeyBinding
  def initialize(key, command, *arguments, no_prefix: false, key_table: nil)
    @key = key
    @command = command
    @arguments = arguments
    @no_prefix = no_prefix
    @key_table = key_table
  end

  def to_s
    result = %w(bind-key)
    result << '-n' if @no_prefix
    result << "-n #{@key_table}" unless @key_table.nil?
    result << @key
    result << @command
    result += @arguments
    result.join(' ')
  end
end

#<> @property prefix_key @args (key) Rebind the tmux prefix key.
property :prefix_key, kind_of: String

#<> @property option @args (name, value=true, scope: '') Set a tmux option with the given scope.
property :option, ::Chef::CookieCutter::FancyProperty,
         collect: true, coerce_class: TmuxOption

#<> @property session_option @args (name, value=true) Set a tmux session option (default scope).
property :session_option, ::Chef::CookieCutter::FancyProperty,
         collect: true, coerce: proc { |name, value| option name, value }

#<> @property window_option @args (name, value=true) Set a tmux window option (scope = 'w').
property :window_option, ::Chef::CookieCutter::FancyProperty,
         collect: true, coerce: proc { |name, value| option name, value, scope: 'w' }

#<> @property server_option @args (name, value=true) Set a tmux server option (scope = 's').
property :server_option, ::Chef::CookieCutter::FancyProperty,
         collect: true, coerce: proc { |name, value| option name, value, scope: 's' }

#<> @property bind_key @args (key, command, arguments..., no_prefix: false, key_table: nil) Bind a key to a command.
property :bind_key, ::Chef::CookieCutter::FancyProperty,
         collect: true, coerce_class: TmuxKeyBinding

#<> @property unbind_key @args (key) Remove an existing keybinding.
property :unbind_key, ::Chef::CookieCutter::FancyProperty,
         collect: true, kind_of: String
