# encoding: UTF-8
#
# Copyright 2015, Ole Claussen <claussen.ole@gmail.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# rubocop:disable Style/BlockDelimiters

#<
# Creates a [Vim](http://www.vim.org) configuration file, by default `~/.vimrc`.
#>

include_resource 'mixin/user_info'
include_resource 'mixin/dotfile'

resource_name :vim_dotfile

provides :dotfile do |node|
  node.run_context.resource_builder.name =~ regex_from_app_name('vim')
end

def initialize(name, run_context = nil)
  super(name, run_context)

  target '.vimrc'
  template_source 'vim/vimrc.erb'
  create_directory ::File.join('.vim', 'autoload')
  create_directory ::File.join('.vim', 'colors')
  create_directory ::File.join('.vim', 'compiler')
  create_directory ::File.join('.vim', 'doc')
  create_directory ::File.join('.vim', 'ftplugin')
  create_directory ::File.join('.vim', 'indent')
  create_directory ::File.join('.vim', 'keymap')
  create_directory ::File.join('.vim', 'lang')
  create_directory ::File.join('.vim', 'plugin')
  create_directory ::File.join('.vim', 'print')
  create_directory ::File.join('.vim', 'spell')
  create_directory ::File.join('.vim', 'syntax')
  create_directory ::File.join('.vim', 'tutor')
end

class VimOption
  attr_reader :name
  attr_reader :value

  def initialize(name, value = true, is_variable: nil)
    @name = name
    @value = value
    @scope = is_variable
  end

  def in_home_directory(home)
    "set #{name}=#{::File.join(home, '.vim', @value)}"
  end

  def to_s
    if !@scope.nil?
      "let #{@scope}:#{@name}=\"#{@value}\""
    elsif @value.is_a?(TrueClass)
      "set #{@name}"
    elsif @value.is_a?(FalseClass)
      "set no#{@name}"
    else
      "set #{@name}=#{@value}"
    end
  end
end

class VimGlobalVariable < VimOption
  def initialize(name, value = true, **_kwargs)
    super(name, value, is_variable: 'g')
  end
end

class VimColors
  attr_reader :name
  attr_reader :git

  def initialize(name, git)
    @name = name
    @git = git
  end
end

#<> @property colors @args (name) Set the Vim colorscheme.
property :colors, kind_of: String

#<> @property pathogen @args (resource) Include Vim Pathogen configuration.
property :pathogen, ::Chef::CookieCutter::FancyProperty,
         coerce_resource: :vim_pathogen_dotfile

#<> @property option @args (name, value=true) Set a Vim option.
property :option, ::Chef::CookieCutter::FancyProperty,
         collect: true, coerce_class: VimOption

#<> @property global_variable @args (name, value=true) Set a global Vim variable.
property :global_variable, ::Chef::CookieCutter::FancyProperty,
         collect: true, coerce_class: VimGlobalVariable

#<> @property directory_option @args (name, directory) Set a Vim option to a directory relative to home, and create that directory.
property :directory_option, ::Chef::CookieCutter::FancyProperty,
         collect: true, coerce_class: VimOption,
         coerce: proc { |option|
           create_directory ::File.join('.vim', option.value)
           option
         }
