# encoding: UTF-8
#
# Copyright 2015, Ole Claussen <claussen.ole@gmail.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# rubocop:disable Style/BlockDelimiters

#<
# Creates an X startup script file, by default `~/.xsession`.
#>

include_resource 'mixin/user_info'
include_resource 'mixin/dotfile'

resource_name :xsession_dotfile

provides :dotfile do |node|
  result = false
  result ||= node.run_context.resource_builder.name =~ regex_from_app_name('xsession')
  result ||= node.run_context.resource_builder.name =~ regex_from_app_name('xinit')
  result
end

def initialize(name, run_context = nil)
  super(name, run_context)

  target '.xsession'
  target '.xinitrc' if name =~ regex_from_app_name('xinit')
  template_source 'xclient/xsession.erb'
  executable true
end

class XResource
  def initialize(name, value)
    @name = name
    @value = value
  end

  def to_s
    "echo \"#{@name}: #{@value}\" | /usr/bin/env xrdb -merge"
  end
end

class XOption
  def initialize(cmd, option, *args)
    @cmd = cmd
    @option = option
    @args = args
  end

  def to_s
    "#{@cmd} #{@option} #{@args.join(' ')}"
  end
end

#<> @property wm_command @args (command) Shell command that represents the session. The session will stop when this command returns.
property :wm_command, kind_of: String, required: true

#<> @property command @args (command) Shell command that should be executed at the start of the session.
property :command, ::Chef::CookieCutter::FancyProperty, collect: true

#<> @property xresources_from_file @args (file) Merge the configuration from the given files into xrdb.
property :xresources_from_file, ::Chef::CookieCutter::FancyProperty, collect: true

#<> @property xresource @args (name, value) Merge a given X resource configuration directly into xrdb.
property :xresource, ::Chef::CookieCutter::FancyProperty,
         collect: true, coerce_class: XResource

#<> @property option @args (command, option, argument...) Set a X option with either the [xsetroot](http://www.x.org/archive/X11R7.5/doc/man/man1/xsetroot.1.html) or [xset](http://www.x.org/archive/X11R7.5/doc/man/man1/xset.1.html) command.
property :option, ::Chef::CookieCutter::FancyProperty,
         collect: true, coerce_class: XOption

#<> @property cursor @args (name) Set the X cursor.
property :cursor, coerce: proc { |name|
  option :xsetroot, '-cursor_name', name
}

#<> @property background_image @args (name) Set the X background image to a bitmap.
property :background_image, coerce: proc { |name|
  option :xsetroot, '-bitmap', name
}

#<> @property background_color @args (color) Set the X background to a solid color.
property :background_color, coerce: proc { |color|
  option :xsetroot, '-solid', "\"#{::Chef::Dotfiles::Colors.from_property(color).to_rgbcolor}\""
}

#<> @property bell @args (arguments) Configure the bell. Arguments can be 'off' or a combination of `volume:`, `pitch:` and `duration:`.
property :bell, coerce: proc { |arg = nil, volume: nil, pitch: nil, duration: nil|
  if arg.is_a?(FalseClass) || arg == 'off'
    option :xset, 'b', 'off'
  else
    args = []
    [volume, pitch, duration].each do |kwarg|
      break if kwarg.nil?
      args << kwarg
    end
    option :xset, 'b', *args
  end
}

#<> @property click @args (arguments) Configure key clicks. Arguments can be 'on', 'off' or a volume in percent.
property :click, coerce: proc { |arg|
  if arg.is_a?(TrueClass) || arg == 'on'
    option :xset, 'c', 'on'
  elsif arg.is_a?(FalseClass) || arg == 'off'
    option :xset, 'c', 'off'
  else
    option :xset, 'c', arg
  end
}

#<> @property dpms @args (arguments) Configure DPMS parameters. Arguments can be 'off', or a combination of `standby:`, `suspend:` and `off:`.
property :dpms, coerce: proc { |arg = nil, standby: nil, suspend: nil, off: nil|
  if arg.is_a?(FalseClass) || arg == 'off'
    option :xset, '-dpms'
  else
    args = []
    [standby, suspend, off].each do |kwarg|
      break if kwarg.nil?
      args << kwarg
    end
    option :xset, 'dpms', *args
  end
}

#<> @property font_path @args (path) Append an element to the current font path.
property :font_path, coerce: proc { |path|
  option :xset, '+fp', path
}

#<> @property mouse @args (acceleration: 2, threshold: 0) Configure mouse parameters.
property :mouse, coerce: proc { |acceleration: 2, threshold: 0|
  option :xset, 'm', acceleration, threshold
}

#<> @property pixel @args (entry:, color:) Configure pixel color values in the color map.
property :pixel, coerce: proc { |entry:, color:|
  option :xset, 'p', entry, ::Chef::Dotfiles::Colors.from_property(color).to_hexcolor
}

#<> @property screen_saver @args (arguments) Configure screen saver parameters. Arguments can be 'on' or 'off'.
property :screen_saver, coerce: proc { |arg|
  if arg.is_a?(TrueClass) || arg == 'on'
    option :xset, 's', 'default'
  elsif arg.is_a?(FalseClass) || arg == 'off'
    option :xset, 's', 'off'
  end
}
